#include <gst/gst.h>
#include <glib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>


#define FILE_PATH "data/movie.mp4"
#define BRANCH_1_FILENAME "data/branch_1_%d.jpeg"
#define BRANCH_2_FILENAME "data/branch_2_%d.jpeg"


int MODE = 0;
GstElement *PIPELINE = NULL;
GMainLoop *MAIN_LOOP = NULL;

void _terminate_pipeline(int signal_no){
    if(PIPELINE != NULL){
        gst_element_send_event(PIPELINE, gst_event_new_eos());
    }
}


void _handle_gst_element_message(GstMessage *message){
    if(!gst_message_has_name(message, "GstMultiFileSink")){
        return;
    }
    GObject *sink = G_OBJECT(message->src);
    int branch_id = (int)g_object_get_data(sink, "branch_id");
    const GstStructure *msg_structure = gst_message_get_structure(message);
    const char *file_name = gst_structure_get_string(msg_structure, "filename");
    g_print("[Branch %d] Crop saved: %s\n", branch_id, file_name);
}


gboolean _handle_pipeline_message(
    GstBus *bus,
    GstMessage *message,
    gpointer data
){  
    switch (GST_MESSAGE_TYPE(message)){
        case GST_MESSAGE_EOS:{
                g_main_loop_quit(MAIN_LOOP);
                break;
            }
            case GST_MESSAGE_ELEMENT:{
                _handle_gst_element_message(message);
                break;
            }
            default:
                break;
        }
    return TRUE;
}


int add_buffer_probe(
    GstElement *element,
    const char *pad_name,
    GstPadProbeCallback probe_callback,
    gpointer user_data
){
    GstPad *pad = gst_element_get_static_pad(element, pad_name);
    if(pad == NULL){
        return -1;
    }
    gulong pre_probe_id = gst_pad_add_probe(
        pad,
        GST_PAD_PROBE_TYPE_BUFFER,
        probe_callback,
        user_data,
        NULL
    );
    if(pre_probe_id == 0){
        return -1;
    }
    return 0;
}

GstElement* initialize_gst_element(
    const char *gst_element_type,
    const char *element_name
){
    GstElement *element = gst_element_factory_make(gst_element_type, element_name);
    if(element == NULL){
        g_printerr(
            "Cannot initialize GST element %s with name %s\n.",
            gst_element_type,
            element_name
        );
        return NULL;
    }
    gboolean result = gst_bin_add(GST_BIN(PIPELINE), element);
    if(result == FALSE){
        g_printerr("Cannot add element %s into bin\n", element_name);
        return NULL;
    }
    return element;
}

void _on_pad_added(GstElement *element, GstPad *pad, void *data){
    /* Gets called when a pad is added to element, connects that
    * new pad to the sink pad of the element in data.
    *
    * This is necessary for using rstpsrc, as it doesn't have any
    * pads when created. */
    GstElement *target_element = GST_ELEMENT(data);
    GstPad *sinkpad = gst_element_get_static_pad(target_element, "sink");
    gst_pad_link(pad, sinkpad);
    gst_object_unref(sinkpad);
}



int connect_dynamic_pipeline_linking_signal_handler(
    GstElement *source,
    GstElement *target
){
    gulong handler_id = g_signal_connect(
        source,
        "pad-added",
        G_CALLBACK(_on_pad_added),
        target);
    if (handler_id <= 0)
    {
        return -1;
    }
    return 0;
}



GstPadProbeReturn monitoring_probe(
    GstPad *pad,
    GstPadProbeInfo *info,
    gpointer u_data
){
    int branch_id = (int)u_data;
    GstElement *queue = (GstElement *)gst_pad_get_parent(pad);
    guint current_buffers, max_buffers, min_buffers, current_bytes, max_bytes;
    guint64 current_times, max_times;
    g_object_get (G_OBJECT(queue),
        "current-level-buffers", &current_buffers,
        "current-level-bytes", &current_bytes,
        "current-level-time", &current_times,
        "max-size-time", &max_times,
        "max-size-buffers", &max_buffers,
        "max-size-bytes", &max_bytes,
        "min-threshold-buffers", &min_buffers,
        NULL
    );
    g_print("[Branch %d] monitoring_probe. Current buffers: %u/%u/%u. Current bytes: %u/%u Current time: %lu/%lu.\n", 
        branch_id, 
        min_buffers, current_buffers, max_buffers,
        current_bytes, max_bytes,
        current_times, max_times
    );
    return GST_PAD_PROBE_OK;
}

GstPadProbeReturn filtering_probe(
    GstPad *pad,
    GstPadProbeInfo *info,
    gpointer u_data
){
    int branch_id = (int)u_data;
    if(MODE == 0){
        // UNFAIR FILTERING
        if(branch_id == 1){
            g_print("UNFAIR FILTERING, branch %d, crop drop.\n", branch_id);
            return GST_PAD_PROBE_DROP;
        } else{
            g_print("UNFAIR FILTERING, branch %d, crop pass.\n", branch_id);
            return GST_PAD_PROBE_OK;
        }
    }
    // RANDOM FILTERING
    if(drand48() < 0.1){
        g_print("RANDOM FILTERING, branch %d, crop pass.\n", branch_id);
        return GST_PAD_PROBE_OK;
    } else {
        g_print("RANDOM FILTERING, branch %d, crop drop.\n", branch_id);
        return GST_PAD_PROBE_DROP;
    }
}



int _assembly_branch(
    GstElement *splitter, 
    GstElement *converter, 
    GstElement *filtering_queue, 
    GstElement *encoder, 
    GstElement *sink,
    int branch_id
){
    if(!gst_element_link_many(splitter, converter, NULL)){
        g_printerr("Could not link source.\n");
        return -1;
    }
    GstCaps *caps = gst_caps_new_simple(
        "video/x-raw",
        "width", G_TYPE_INT, 150,
        "height", G_TYPE_INT, 150,
        NULL
    );
    if(!gst_element_link_filtered(converter, filtering_queue, caps)){
        g_printerr("Elements (converter, filtering_queue) could not be linked.\n");
        gst_caps_unref(caps);
        return -1;
    }
    gst_caps_unref(caps);
    if(!gst_element_link_many(filtering_queue, encoder, sink, NULL)){
        g_printerr("Cannot link target endocer with sink in crop context.\n");
        return -1;
    }
    g_object_set(
        G_OBJECT(converter),
        "src-crop", "500:500:150:150",
        "output-buffers", 64,
        NULL
    );
    g_object_set(
        G_OBJECT(filtering_queue),
        "max-size-time", 0,
        "max-size-buffers", 0,
        "max-size-bytes", 0,
        NULL
    );
    char *filename_buffer;
    if (branch_id == 1){
        filename_buffer = BRANCH_1_FILENAME;
    } else {
        filename_buffer = BRANCH_2_FILENAME;
    }
    g_object_set(
        G_OBJECT(sink),
        "location", filename_buffer,
        "post-messages", 1,
        NULL
    );
    g_object_set_data(
        G_OBJECT(sink),
        "branch_id",
        branch_id
    );
    add_buffer_probe(
        filtering_queue,
        "sink",
        monitoring_probe,
        branch_id
    );
    add_buffer_probe(
        filtering_queue,
        "src",
        filtering_probe,
        branch_id
    );
    return 0;
}


int _assembly_pipeliine(){
    GstElement *source = initialize_gst_element("filesrc", "source");
    GstElement *demux = initialize_gst_element("qtdemux", "demux");
    GstElement *source_queue = initialize_gst_element("queue", "source_queue");
    GstElement *parser = initialize_gst_element("h264parse", "parser");
    GstElement *decoder = initialize_gst_element("nvv4l2decoder", "decoder");
    GstElement *splitter = initialize_gst_element("tee", "splitter");
    GstElement *converter_1 = initialize_gst_element("nvvideoconvert", "converter_1");
    GstElement *converter_2 = initialize_gst_element("nvvideoconvert", "converter_2");
    GstElement *filtering_queue_1 = initialize_gst_element("queue", "filtering_queue_1");
    GstElement *filtering_queue_2 = initialize_gst_element("queue", "filtering_queue_2");
    GstElement *encoder_1 = initialize_gst_element("jpegenc", "encoder_1");
    GstElement *encoder_2 = initialize_gst_element("jpegenc", "encoder_2");
    GstElement *sink_1 = initialize_gst_element("multifilesink", "sink_1");
    GstElement *sink_2 = initialize_gst_element("multifilesink", "sink_2");
    if(
        source == NULL || demux == NULL || source_queue == NULL || parser == NULL || decoder == NULL || splitter == NULL || 
        converter_1 == NULL || converter_2 == NULL || filtering_queue_1 == NULL || filtering_queue_2 == NULL ||
        encoder_1 == NULL || encoder_2 == NULL || sink_1 == NULL || sink_2 == NULL
    ){
        return -1;
    }
    if(!gst_element_link_many(source, demux, NULL)){
        g_printerr("Could not link source.\n");
        return -1;
    }
    if(connect_dynamic_pipeline_linking_signal_handler(demux, source_queue) != 0){
        g_printerr("Could not link demux.\n");
        return -1;
    }
    if(!gst_element_link_many(source_queue, parser, decoder, splitter, NULL)){
        g_printerr("Could not link common part of pipeline.\n");
        return -1;
    }
    if(
        _assembly_branch(splitter, converter_1, filtering_queue_1, encoder_1, sink_1, 1) != 0 ||
        _assembly_branch(splitter, converter_2, filtering_queue_2, encoder_2, sink_2, 2) != 0
    ){
        g_printerr("Could not link crop branches.\n");
        return -1;
    }
    g_object_set(
        G_OBJECT(source),
        "location", FILE_PATH,
        NULL
    );
    return 0;
}

int main(int argc, char **argv){
    if(argc > 1){
        MODE = atoi(argv[1]);
        g_print("MODE: %d\n", MODE);
    }
    gst_init(NULL, NULL);
    srand48(time(NULL));
    MAIN_LOOP = g_main_loop_new(NULL, FALSE);
    if (MAIN_LOOP == NULL){
        g_printerr("Cannot initialize main loop.");
        return 1;
    }
    PIPELINE = gst_pipeline_new("test-pipeline");
    if (PIPELINE == NULL){
        g_printerr("Cannot initialize pipeline.");
        return 1;
    }
    GstBus *gst_bus = gst_pipeline_get_bus(GST_PIPELINE(PIPELINE));
    if (gst_bus == NULL){
        g_printerr("Cannot initialize message bus.");
        return 1;
    }
    gst_bus_add_watch(gst_bus, _handle_pipeline_message, NULL);
    signal(SIGINT, _terminate_pipeline);
    if(_assembly_pipeliine() != 0){
        return 2;
    }
    if (gst_element_set_state(PIPELINE, GST_STATE_READY) == GST_STATE_CHANGE_FAILURE){
        g_printerr("Unable to set the pipeline to the ready state.\n");
        return 3;
    }
    g_print("Ready state...\n");
    if (gst_element_set_state(PIPELINE, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE){
        g_printerr("Unable to set the pipeline to the playing state.\n");
        return 3;
    }
    GST_DEBUG_BIN_TO_DOT_FILE(GST_BIN(PIPELINE), GST_DEBUG_GRAPH_SHOW_ALL, "pipeline");
    g_print("Running...\n");
    g_main_loop_run(MAIN_LOOP);
    gst_element_set_state(PIPELINE, GST_STATE_NULL);
    return 0;
}